<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Question Writer text format importer.
 *
 * @package qformat_questionwritertext
 * @copyright 2010 Tim Hunt
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


require_once($CFG->dirroot . '/question/format.php');


/**
 * The Question Writer text format class.
 *
 * At the moment this class only handles Multiple-choice questions. A multichoice
 * question in this format looks like this:
 * 
 * Type: MC
 * 1) A water or foodborne disease outbreak with a short incubation period is likely because of
 * *a. chemical poisoning
 * b. schistosomiasis
 * c. salmonellosis
 * d. shigella
 *
 * @copyright 2010 Tim Hunt
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qformat_questionwritertext extends qformat_default {

    function provide_import() {
        return true;
    }

    function readquestion($lines) {
        $typeline = trim(array_shift($lines));
        if (strpos($typeline, 'Type: MC') === false) {
            $this->error(get_string('canonlyimportmc', 'qformat_questionwritertext'), $typeline);
            return false;
        }

        $question = $this->defaultquestion();

        $question->questiontext = preg_replace('/^\d+\)\s+/', '', trim(array_shift($lines)));
        $question->questiontextformat = FORMAT_PLAIN;
        $question->name = shorten_text($question->questiontext, 80);
        $question->qtype = 'multichoice';
        $question->single = true;
        $question->shuffleanswers = false;

        $question->answer = array();
        $question->fraction = array();
        $question->feedback = array();
        $rightanswerfound = false;
        foreach ($lines as $line) {
            if (!preg_match('/^(\*?)[a-z]\.\s*(.+)$/', trim($line), $matches)) {
                continue;
            }

            $question->answer[] = $matches[2];
            if ($matches[1]) {
                $question->fraction[] = 1;
                $rightanswerfound = true;
            } else {
                $question->fraction[] = 0;
            }
            $question->feedback[] = '';
        }

        if (!$rightanswerfound) {
            $this->error(get_string('mustspecifyrightanswer', 'qformat_questionwritertext'),
                    $question->questiontext);
            return false;
        }

        if (count($question->answer) < 2) {
            $this->error(get_string('notenoughchoices', 'qformat_questionwritertext', count($question->answer)),
                    $question->questiontext);
            return false;
        }

        return $question;
    }
}
