<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Unit tests for the Question Writer text import format.
 *
 * @package qformat_questionwritertext
 * @copyright 2010 Tim Hunt
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once($CFG->libdir . '/questionlib.php');
require_once($CFG->dirroot . '/question/format/questionwritertext/format.php');


/**
 * Unit tests for the Question Writer test impoorter.
 *
 * @copyright 2010 Tim Hunt
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qformat_questionwritertext_test extends UnitTestCase {
    public function test_import_multichoice() {
        $lines = array(
            'Type: MC',
            '1) A water or foodborne disease outbreak with a short incubation period is likely because of',
            'a. schistosomiasis',
            '*b. chemical poisoning',
            'c. salmonellosis',
            'd. shigella',
            '',
            'Type: MC',
            '8) It is believed that the ingestion of one virus particle can cause infection in humans.',
            '*a. True',
            'b. False',
        );

        $importer = new qformat_questionwritertext();
        $questions = $importer->readquestions($lines);

        $this->assertEqual(2, count($questions));

        $modelquestion1 = new stdClass;
        $modelquestion1->shuffleanswers = false;
        $modelquestion1->defaultgrade = 1;
        $modelquestion1->image = '';
        $modelquestion1->usecase = 0;
        $modelquestion1->multiplier = array();
        $modelquestion1->generalfeedback = '';
        $modelquestion1->correctfeedback = '';
        $modelquestion1->partiallycorrectfeedback = '';
        $modelquestion1->incorrectfeedback = '';
        $modelquestion1->answernumbering = 'abc';
        $modelquestion1->penalty = 0.1;
        $modelquestion1->length = 1;
        $modelquestion1->export_process = 1;
        $modelquestion1->import_process = 1;
        $modelquestion1->questiontext = 'A water or foodborne disease outbreak with a short incubation period is likely because of';
        $modelquestion1->questiontextformat = 2;
        $modelquestion1->name = 'A water or foodborne disease outbreak with a short incubation period is ...';
        $modelquestion1->qtype = 'multichoice';
        $modelquestion1->single = 1;
        $modelquestion1->answer = array(
        'schistosomiasis',
        'chemical poisoning',
        'salmonellosis',
        'shigella');
        $modelquestion1->fraction = Array(0, 1, 0, 0);
        $modelquestion1->feedback = array('', '', '', '');
        $this->assertEqual($modelquestion1, $questions[0]);

        $modelquestion2 = new stdClass;
        $modelquestion2->shuffleanswers = false;
        $modelquestion2->defaultgrade = 1;
        $modelquestion2->image = '';
        $modelquestion2->usecase = 0;
        $modelquestion2->multiplier = array();
        $modelquestion2->generalfeedback = '';
        $modelquestion2->correctfeedback = '';
        $modelquestion2->partiallycorrectfeedback = '';
        $modelquestion2->incorrectfeedback = '';
        $modelquestion2->answernumbering = 'abc';
        $modelquestion2->penalty = 0.1;
        $modelquestion2->length = 1;
        $modelquestion2->export_process = 1;
        $modelquestion2->import_process = 1;
        $modelquestion2->questiontext = 'It is believed that the ingestion of one virus particle can cause infection in humans.';
        $modelquestion2->questiontextformat = 2;
        $modelquestion2->name = 'It is believed that the ingestion of one virus particle can cause infection ...';
        $modelquestion2->qtype = 'multichoice';
        $modelquestion2->single = 1;
        $modelquestion2->answer = array('True', 'False');
        $modelquestion2->fraction = array(1, 0);
        $modelquestion2->feedback = array('', '');

        $this->assertEqual($modelquestion2, $questions[1]);
    }

    public function test_import_unknown_type_reported() {
        $lines = array(
            'Type: frog',
        );

        ob_start();
        $importer = new qformat_questionwritertext();
        $questions = $importer->readquestions($lines);
        $error = ob_get_contents();
        ob_end_clean();

        $this->assertPattern('/' . preg_quote(get_string(
                'canonlyimportmc', 'qformat_questionwritertext')) . '/', $error);
    }

    public function test_import_multichoice_not_enough_choices_reported() {
        $lines = array(
            'Type: MC',
            '1) A water or foodborne disease outbreak with a short incubation period is likely because of',
            '*a. chemical poisoning',
        );

        ob_start();
        $importer = new qformat_questionwritertext();
        $questions = $importer->readquestions($lines);
        $error = ob_get_contents();
        ob_end_clean();

        $this->assertPattern('/' . preg_quote(get_string(
                'notenoughchoices', 'qformat_questionwritertext', 1)) . '/', $error);
    }

    public function test_import_multichoice_no_right_choice_reported() {
        $lines = array(
            'Type: MC',
            '1) A water or foodborne disease outbreak with a short incubation period is likely because of',
            'a. chemical poisoning',
            'b. schistosomiasis',
            'c. salmonellosis',
            'd. shigella',
        );

        ob_start();
        $importer = new qformat_questionwritertext();
        $questions = $importer->readquestions($lines);
        $error = ob_get_contents();
        ob_end_clean();

        $this->assertPattern('/' . preg_quote(get_string(
                'mustspecifyrightanswer', 'qformat_questionwritertext')) . '/', $error);
    }
}